@@ -48,6 +48,10 @@
 #endif
 #endif
 
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 #include "gioalias.h"
 
 #define g_local_file_output_stream_get_type _g_local_file_output_stream_get_type
@@ -199,7 +203,7 @@ g_local_file_output_stream_close (GOutputStream  *stream,
 	  
 #ifdef HAVE_LINK
 	  /* create original -> backup link, the original is then renamed over */
-	  if (unlink (file->priv->backup_filename) != 0 &&
+	  if (g_unlink (file->priv->backup_filename) != 0 &&
 	      errno != ENOENT)
 	    {
               int errsv = errno;
@@ -214,7 +218,7 @@ g_local_file_output_stream_close (GOutputStream  *stream,
 	  if (link (file->priv->original_filename, file->priv->backup_filename) != 0)
 	    {
 	      /*  link failed or is not supported, try rename  */
-	      if (rename (file->priv->original_filename, file->priv->backup_filename) != 0)
+	      if (g_rename (file->priv->original_filename, file->priv->backup_filename) != 0)
 		{
                   int errsv = errno;
 
@@ -227,7 +231,7 @@ g_local_file_output_stream_close (GOutputStream  *stream,
 	    }
 #else
 	    /* If link not supported, just rename... */
-	  if (rename (file->priv->original_filename, file->priv->backup_filename) != 0)
+	  if (g_rename (file->priv->original_filename, file->priv->backup_filename) != 0)
 	    {
               int errsv = errno;
 
@@ -245,7 +249,7 @@ g_local_file_output_stream_close (GOutputStream  *stream,
 	goto err_out;
 
       /* tmp -> original */
-      if (rename (file->priv->tmp_filename, file->priv->original_filename) != 0)
+      if (g_rename (file->priv->tmp_filename, file->priv->original_filename) != 0)
 	{
           int errsv = errno;
 
@@ -456,7 +460,7 @@ _g_local_file_output_stream_create  (const char        *filename,
   else
     mode = 0666;
   
-  fd = g_open (filename, O_CREAT | O_EXCL | O_WRONLY, mode);
+  fd = g_open (filename, O_CREAT | O_EXCL | O_WRONLY | O_BINARY, mode);
   if (fd == -1)
     {
       int errsv = errno;
@@ -467,10 +471,14 @@ _g_local_file_output_stream_create  (const char        *filename,
 		     G_IO_ERROR_INVALID_FILENAME,
 		     _("Invalid filename"));
       else
-	g_set_error (error, G_IO_ERROR,
-		     g_io_error_from_errno (errsv),
-		     _("Error opening file '%s': %s"),
-		     filename, g_strerror (errsv));
+	{
+	  char *display_name = g_filename_display_name (filename);
+	  g_set_error (error, G_IO_ERROR,
+		       g_io_error_from_errno (errsv),
+		       _("Error opening file '%s': %s"),
+		       display_name, g_strerror (errsv));
+	  g_free (display_name);
+	}
       return NULL;
     }
   
@@ -497,7 +505,7 @@ _g_local_file_output_stream_append  (const char        *filename,
   else
     mode = 0666;
 
-  fd = g_open (filename, O_CREAT | O_APPEND | O_WRONLY, mode);
+  fd = g_open (filename, O_CREAT | O_APPEND | O_WRONLY | O_BINARY, mode);
   if (fd == -1)
     {
       int errsv = errno;
@@ -508,10 +516,14 @@ _g_local_file_output_stream_append  (const char        *filename,
 		     G_IO_ERROR_INVALID_FILENAME,
 		     _("Invalid filename"));
       else
-	g_set_error (error, G_IO_ERROR,
-		     g_io_error_from_errno (errsv),
-		     _("Error opening file '%s': %s"),
-		     filename, g_strerror (errsv));
+	{
+	  char *display_name = g_filename_display_name (filename);
+	  g_set_error (error, G_IO_ERROR,
+		       g_io_error_from_errno (errsv),
+		       _("Error opening file '%s': %s"),
+		       display_name, g_strerror (errsv));
+	  g_free (display_name);
+	}
       return NULL;
     }
   
@@ -611,9 +623,9 @@ handle_overwrite_open (const char    *filename,
   /* We only need read access to the original file if we are creating a backup.
    * We also add O_CREATE to avoid a race if the file was just removed */
   if (create_backup)
-    open_flags = O_RDWR | O_CREAT;
+    open_flags = O_RDWR | O_CREAT | O_BINARY;
   else
-    open_flags = O_WRONLY | O_CREAT;
+    open_flags = O_WRONLY | O_CREAT | O_BINARY;
   
   /* Some systems have O_NOFOLLOW, which lets us avoid some races
    * when finding out if the file we opened was a symlink */
@@ -636,22 +648,24 @@ handle_overwrite_open (const char    *filename,
   if (fd == -1)
     {
       int errsv = errno;
-
+      char *display_name = g_filename_display_name (filename);
       g_set_error (error, G_IO_ERROR,
 		   g_io_error_from_errno (errsv),
 		   _("Error opening file '%s': %s"),
-		   filename, g_strerror (errsv));
+		   display_name, g_strerror (errsv));
+      g_free (display_name);
       return -1;
     }
   
   if (fstat (fd, &original_stat) != 0) 
     {
       int errsv = errno;
-
+      char *display_name = g_filename_display_name (filename);
       g_set_error (error, G_IO_ERROR,
 		   g_io_error_from_errno (errsv),
 		   _("Error stating file '%s': %s"),
-		   filename, g_strerror (errsv));
+		   display_name, g_strerror (errsv));
+      g_free (display_name);
       goto err_out;
     }
   
@@ -716,10 +730,10 @@ handle_overwrite_open (const char    *filename,
       /* try to keep permissions */
 
       if (
-#ifdef F_CHOWN
+#ifdef HAVE_FCHOWN
 	  fchown (tmpfd, original_stat.st_uid, original_stat.st_gid) == -1 ||
 #endif
-#ifdef F_CHMOD
+#ifdef HAVE_FCHMOD
 	  fchmod (tmpfd, original_stat.st_mode) == -1 ||
 #endif
 	  0
@@ -734,7 +748,7 @@ handle_overwrite_open (const char    *filename,
 	      original_stat.st_mode != tmp_statbuf.st_mode)
 	    {
 	      close (tmpfd);
-	      unlink (tmp_filename);
+	      g_unlink (tmp_filename);
 	      g_free (tmp_filename);
 	      goto fallback_strategy;
 	    }
@@ -755,7 +769,7 @@ handle_overwrite_open (const char    *filename,
       
       backup_filename = create_backup_filename (filename);
 
-      if (unlink (backup_filename) == -1 && errno != ENOENT)
+      if (g_unlink (backup_filename) == -1 && errno != ENOENT)
 	{
 	  g_set_error (error,
 		       G_IO_ERROR,
@@ -766,7 +780,7 @@ handle_overwrite_open (const char    *filename,
 	}
 
       bfd = g_open (backup_filename,
-		    O_WRONLY | O_CREAT | O_EXCL,
+		    O_WRONLY | O_CREAT | O_EXCL | O_BINARY,
 		    original_stat.st_mode & 0777);
 
       if (bfd == -1)
@@ -783,14 +797,14 @@ handle_overwrite_open (const char    *filename,
        * original file. If this fails, set the protection
        * bits for the group same as the protection bits for
        * others. */
-#ifdef HAVE_FCHOWN
+#if defined(HAVE_FCHOWN) && defined(HAVE_FCHMOD)
       if (fstat (bfd, &tmp_statbuf) != 0)
 	{
 	  g_set_error (error,
 		       G_IO_ERROR,
 		       G_IO_ERROR_CANT_CREATE_BACKUP,
 		       _("Backup file creation failed"));
-	  unlink (backup_filename);
+	  g_unlink (backup_filename);
 	  g_free (backup_filename);
 	  goto err_out;
 	}
@@ -806,7 +820,7 @@ handle_overwrite_open (const char    *filename,
 			   G_IO_ERROR,
 			   G_IO_ERROR_CANT_CREATE_BACKUP,
 			   _("Backup file creation failed"));
-	      unlink (backup_filename);
+	      g_unlink (backup_filename);
 	      close (bfd);
 	      g_free (backup_filename);
 	      goto err_out;
@@ -820,7 +834,7 @@ handle_overwrite_open (const char    *filename,
 		       G_IO_ERROR,
 		       G_IO_ERROR_CANT_CREATE_BACKUP,
 		       _("Backup file creation failed"));
-	  unlink (backup_filename);
+	  g_unlink (backup_filename);
 	  close (bfd);
 	  g_free (backup_filename);
 	  
@@ -890,7 +904,7 @@ _g_local_file_output_stream_replace (const char        *filename,
     mode = 0666;
 
   /* If the file doesn't exist, create it */
-  fd = g_open (filename, O_CREAT | O_EXCL | O_WRONLY, mode);
+  fd = g_open (filename, O_CREAT | O_EXCL | O_WRONLY | O_BINARY, mode);
 
   if (fd == -1 && errno == EEXIST)
     {
@@ -910,10 +924,14 @@ _g_local_file_output_stream_replace (const char        *filename,
 		     G_IO_ERROR_INVALID_FILENAME,
 		     _("Invalid filename"));
       else
-	g_set_error (error, G_IO_ERROR,
-		     g_io_error_from_errno (errsv),
-		     _("Error opening file '%s': %s"),
-		     filename, g_strerror (errsv));
+	{
+	  char *display_name = g_filename_display_name (filename);
+	  g_set_error (error, G_IO_ERROR,
+		       g_io_error_from_errno (errsv),
+		       _("Error opening file '%s': %s"),
+		       display_name, g_strerror (errsv));
+	  g_free (display_name);
+	}
       return NULL;
     }
   
